package pl.polgrabiat.websockets.chat.websocketschat.handlers;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;
import org.springframework.web.socket.CloseStatus;
import org.springframework.web.socket.TextMessage;
import org.springframework.web.socket.WebSocketSession;
import org.springframework.web.socket.handler.TextWebSocketHandler;
import pl.polgrabiat.websockets.chat.websocketschat.dto.GlobalCtx;
import pl.polgrabiat.websockets.chat.websocketschat.dto.UserCtx;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

@Component
public class ChatWebsocketHandler extends TextWebSocketHandler {

    private static final Logger lg = LoggerFactory.getLogger(ChatWebsocketHandler.class);

    private Set<WebSocketSession> sessions = new HashSet<>();
    private Map<String, SessionCommandHandler> commandHandlers = new HashMap<>();
    private GlobalCtx globalCtx = new GlobalCtx();

    public ChatWebsocketHandler() {
        commandHandlers.put("USER", new UserCommandHandler());
        commandHandlers.put("NICK", new NickCommandHandler());
        commandHandlers.put("JOIN", new JoinCommandHandler());
        commandHandlers.put("LEAVE", new LeaveCommandHandler());
        commandHandlers.put("PRIVMSG", new PrivMessageHandler());
    }

    @Override
    public void afterConnectionEstablished(WebSocketSession session) throws Exception {
        super.afterConnectionEstablished(session);
        sessions.add(session);
        globalCtx.getUserSessions().put(session, new UserCtx(session));
    }

    @Override
    public void afterConnectionClosed(WebSocketSession session, CloseStatus status) throws Exception {
        super.afterConnectionClosed(session, status);
        sessions.remove(session);
        globalCtx.getUserSessions().remove(session);
    }

    @Override
    protected void handleTextMessage(WebSocketSession session, TextMessage message) throws Exception {
        super.handleTextMessage(session, message);

        lg.info("Message received: {}", message);
        String payload = message.getPayload();
        int commandEndIdx = payload.indexOf(" ");
        if (commandEndIdx < 0) {
            // no command
            session.sendMessage(new TextMessage("Invalid command " + payload));
            return;
        }
        String command = payload.substring(0, commandEndIdx);
        String data = payload.substring(commandEndIdx+1);
        SessionCommandHandler commandHandler = commandHandlers.get(command.toUpperCase());
        if (commandHandler != null) {
            commandHandler.handleCommand(globalCtx, session, payload, command, data);
            return;
        }

        session.sendMessage(new TextMessage("Invalid command " + payload));
    }
}
